@@ -368,11 +368,6 @@ inline void buffer_read(lkst_buffer_id id, unsigned int num, char *file_name,
 		lbuffer.buffer = buffer;
 
 		retval = ioctl(fd, LKST_IOC_BUFFER_READ, &lbuffer);
-							/* buffer read */
-		printf("ioctl read buffer id=%d\n", lbuffer.id);
-		printf("ioctl retval=%d result_read_size=%d\n", retval,
-		       lbuffer.result_read_size);
-
 		if (retval){
 			fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
 			printf("Skip reading this buffer!\n");
@@ -435,6 +430,7 @@ inline void buffer_print_wonly(char*  program_name,    /* name of Execution file
 				int   arg_filter_flag, /* 1=(-e option published) or 0=(not) */
 				int   arg_entry_num,
 				int   arg_sort_flag,
+				int   arg_csv_flag,
 				char* arg_inputfile,   /* name of reading file */
 				char* event_work)      /* The argument of "-e" option */
 {
@@ -499,7 +495,7 @@ inline void buffer_print_wonly(char*  program_name,    /* name of Execution file
 	pointpri.fdata           = pointsor.fdata;
 	pointpri.evr             = pointsor.evr;
 	/* call function */
-	return_num = print_data(&(pointpri));
+	return_num = print_data(&(pointpri),arg_csv_flag);
 	if (return_num < LKST_LOG_FMT_RET_CODE_TRUE) {
 		printf("%s: error in function-print_data\n\n", program_name);
 		exit(1);
@@ -515,6 +511,7 @@ inline void buffer_print_randw(lkst_buffer_id  id,
 				char*          program_name,
 				int            arg_cpu_id,
 				int            arg_filter_flag,
+				int	       arg_csv_flag,
 				int            arg_entry_num,
 				int            arg_sort_flag,
 				char           event_work[])
@@ -585,7 +582,7 @@ inline void buffer_print_randw(lkst_buffer_id  id,
 	pointpri.fdata           = pointsor.fdata;
 	pointpri.evr             = pointsor.evr;
 	/* call function */
-	return_num = print_data(&(pointpri));
+	return_num = print_data(&(pointpri),arg_csv_flag);
 	if (return_num < LKST_LOG_FMT_RET_CODE_TRUE) {
 		printf("%s: error in function-print_data\n\n", program_name);
 		exit(1);
@@ -1076,7 +1073,7 @@ signed long cfnc_descending(const void *pointer1, const void *pointer2)
 	return (signed long)(w2->posix.log_time.tv_sec) - (signed long)(w1->posix.log_time.tv_sec);
 }
 
-int print_data(struct lkst_arg_print_data *pointpri)
+int print_data(struct lkst_arg_print_data *pointpri,int arg_csv_flag)
 {
 	int                    i, j;                                  /* roop flag */
 	int                    out_Entry_num;                         /* The number of output items */
@@ -1123,42 +1120,75 @@ int print_data(struct lkst_arg_print_data *pointpri)
 
 		if (strcmp( event_Name_str, "" )==0) {
 			/* event_data undefined ? */
-			printf("event_type=0x%08x\n", event_type_num);
+			if ( arg_csv_flag )
+				printf("0x%08x,", event_type_num);
+			else
+				printf("event_type=0x%08x\n", event_type_num);
 		} else {
 			/* display  event name */
-			printf("event_type=%s\n", event_Name_str);
+			if ( arg_csv_flag )
+				printf("\"%s\",", event_Name_str);
+			else
+				printf("event_type=%s\n", event_Name_str);
 		}
 		/* display cpu pid time */
-		printf("\tcpu=%02lu, pid=%08lu\n", evr[i]->posix.log_processor, evr[i]->posix.log_pid);
-		clock_viewer((unsigned long)evr[i]->posix.log_time.tv_sec, (unsigned long)evr[i]->posix.log_time.tv_nsec);
+		if ( arg_csv_flag )
+			printf("%02lu,%08lu,", evr[i]->posix.log_processor, evr[i]->posix.log_pid);
+		else
+			printf("\tcpu=%02lu, pid=%08lu\n", evr[i]->posix.log_processor, evr[i]->posix.log_pid);
+		clock_viewer((unsigned long)evr[i]->posix.log_time.tv_sec, (unsigned long)evr[i]->posix.log_time.tv_nsec,arg_csv_flag);
 		/* display args_data */
 		if (( LKST_event_data[event_type_num].LKST_inf_args[0] != 0x00) &&
 		    (strcmp( LKST_event_data[event_type_num].LKST_inf_args[0], "") != 0)) {
-			printf("\targ1=0x%08x 0x%08x : %s\n",
-				*((unsigned long *)(&(evr[i]->log_arg1))+1),
-				*((unsigned long *)(&(evr[i]->log_arg1))),
-				LKST_event_data[event_type_num].LKST_inf_args[0]);
+			if ( arg_csv_flag )
+				printf("\"%s\",0x%08x,0x%08x,",
+					LKST_event_data[event_type_num].LKST_inf_args[0],
+					*((unsigned long *)(&(evr[i]->log_arg1))+1),
+					*((unsigned long *)(&(evr[i]->log_arg1))));
+			else	
+				printf("\targ1=0x%08x 0x%08x : %s\n",
+					*((unsigned long *)(&(evr[i]->log_arg1))+1),
+					*((unsigned long *)(&(evr[i]->log_arg1))),
+					LKST_event_data[event_type_num].LKST_inf_args[0]);
 		}
 		if (( LKST_event_data[event_type_num].LKST_inf_args[1] != 0x00) &&
 		    (strcmp( LKST_event_data[event_type_num].LKST_inf_args[1], "") != 0)) {
-			printf("\targ2=0x%08x 0x%08x : %s\n",
-				*((unsigned long *)(&(evr[i]->log_arg2))+1),
-				*((unsigned long *)(&(evr[i]->log_arg2))),
-				LKST_event_data[event_type_num].LKST_inf_args[1]);
+			if ( arg_csv_flag )
+				printf("\"%s\",0x%08x,0x%08x,",
+					LKST_event_data[event_type_num].LKST_inf_args[1],
+					*((unsigned long *)(&(evr[i]->log_arg2))+1),
+					*((unsigned long *)(&(evr[i]->log_arg2))));
+			else
+				printf("\targ2=0x%08x 0x%08x : %s\n",
+					*((unsigned long *)(&(evr[i]->log_arg2))+1),
+					*((unsigned long *)(&(evr[i]->log_arg2))),
+					LKST_event_data[event_type_num].LKST_inf_args[1]);
 		}
 		if (( LKST_event_data[event_type_num].LKST_inf_args[2] != 0x00) &&
 		    (strcmp( LKST_event_data[event_type_num].LKST_inf_args[2], "") != 0)) {
-			printf("\targ3=0x%08x 0x%08x : %s\n",
-				*((unsigned long *)(&(evr[i]->log_arg3))+1),
-				*((unsigned long *)(&(evr[i]->log_arg3))),
-				LKST_event_data[event_type_num].LKST_inf_args[2]);
+			if ( arg_csv_flag )
+				printf("\"%s\",0x%08x,0x%08x,",
+					LKST_event_data[event_type_num].LKST_inf_args[2],
+					*((unsigned long *)(&(evr[i]->log_arg3))+1),
+					*((unsigned long *)(&(evr[i]->log_arg3))));
+			else
+				printf("\targ3=0x%08x 0x%08x : %s\n",
+					*((unsigned long *)(&(evr[i]->log_arg3))+1),
+					*((unsigned long *)(&(evr[i]->log_arg3))),
+					LKST_event_data[event_type_num].LKST_inf_args[2]);
 		}
 		if (( LKST_event_data[event_type_num].LKST_inf_args[3] != 0x00) &&
 		    (strcmp( LKST_event_data[event_type_num].LKST_inf_args[3], "") != 0)) {
-			printf("\targ4=0x%08x 0x%08x : %s\n",
-				*((unsigned long *)(&(evr[i]->log_arg4))+1),
-				*((unsigned long *)(&(evr[i]->log_arg4))),
-				LKST_event_data[event_type_num].LKST_inf_args[3]);
+			if ( arg_csv_flag )
+				printf("\"%s\",0x%08x,0x%08x",
+					LKST_event_data[event_type_num].LKST_inf_args[3],
+					*((unsigned long *)(&(evr[i]->log_arg4))+1),
+					*((unsigned long *)(&(evr[i]->log_arg4))));
+			else
+				printf("\targ4=0x%08x 0x%08x : %s\n",
+					*((unsigned long *)(&(evr[i]->log_arg4))+1),
+					*((unsigned long *)(&(evr[i]->log_arg4))),
+					LKST_event_data[event_type_num].LKST_inf_args[3]);
 		}
 		printf("\n");
 		out_Entry_num++;
@@ -1195,15 +1225,29 @@ void help_viewer(char* program_name)
 	printf("                         (default : descending sort)\n");
 }
 
-void clock_viewer(unsigned long time_sec, unsigned long time_usec)
+void clock_viewer(unsigned long time_sec, unsigned long time_usec,int arg_csv_form)
 {
 	struct tm *newtime;
 	time_t    ltime;
 	char      monthName[12][4]  ={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
-	char      weekdayName[7][4] ={"Sun","Mon","Tue","Wed","Fri","Sat"};
+	char      weekdayName[7][4] ={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
 
 	ltime = (time_t)time_sec;
 	newtime = localtime( &ltime );
+	if ( arg_csv_form )
+	{
+	printf( "%s,%s,%02d,%02d:%02d:%02d.%lu,%04d,",
+		weekdayName[newtime->tm_wday],
+		monthName[newtime->tm_mon],
+		newtime->tm_mday,
+		newtime->tm_hour,
+		newtime->tm_min,
+		newtime->tm_sec,
+		time_usec,
+		newtime->tm_year + LKST_LOG_FMT_PRINT_SIZE);
+	}
+	else
+	{
 	printf( "\ttime=%s %s %02d %02d:%02d:%02d.%lu %04d\n",
 		weekdayName[newtime->tm_wday],
 		monthName[newtime->tm_mon],
@@ -1213,6 +1257,7 @@ void clock_viewer(unsigned long time_sec, unsigned long time_usec)
 		newtime->tm_sec,
 		time_usec,
 		newtime->tm_year + LKST_LOG_FMT_PRINT_SIZE);
+	}
 }
 
 #if DEBUG
