@@ -17,13 +17,16 @@
 #include <sys/wait.h>
 #include <errno.h>
 #include <string.h>
+#include <stdarg.h>
+#include <syslog.h>
+
 
 #include "command.h"
 #define LKST_ETYPE_DEF_DESCRIPTIONS
 #include <linux/lkst_events.h>
 
 #define NAME	"lkstlogd"
-#define VERSION "1.2"
+#define VERSION "1.4"
 
 #define LOG_HEADER_SIZE \
 	(sizeof(log_header_t) +					\
@@ -42,23 +45,24 @@ int  active_cpu(int, int *);
 int  buf_read(int, struct lkst_log_record*, int);
 int  buffer_read_size(int);
 void cleanup();
-void sighup_handler();
+
+void sighup_handler(int);
 void die_handler(int);
 void child_die_handler(int);
-void par_snap_handler();
-void par_switch_handler();
-void child_switch_handler();
-void child_end_snap_handler();
-void child_snap_handler();
+void par_snap_handler(int);
+void par_switch_handler(int);
+void child_switch_handler(int);
+void child_snap_flip_handler(int);
 int  kill_children(int);
 int  del_pid_file();
+
 void usage(void);
 void buffer_setrmod(int);
 void buffer_ring(int, int);
 void init_maskset(int);
 void init_buffer(int);
-void restore_maskset(void);
-void restore_buffer(void);
+void restore_maskset(int);
+void restore_buffer(int);
 void write_header(FILE*);
 
 int read_on = 0;
@@ -74,9 +78,9 @@ int endian_big;
 int buf_ver;
 char arch[LKST_ARCH_NAME_LEN];
 
-FILE *logfd ;
-FILE *logfd_toclose=NULL ;
-FILE *logfd_c ;
+FILE *logfd = NULL;
+FILE *logfd_toclose = NULL ;
+FILE *logfd_c = NULL;
 
 #ifndef LKSTLOGD_PID
 # ifdef DEBUG
@@ -106,8 +110,11 @@ static char log_file[256] = LKSTLOGD_LOGFILE;
 int init_buf_size = INIT_BUF_SIZE_DEFAULT;
 int init_buf_num = INIT_BUF_NUM_DEFAULT;
 
-int ncpu;	// number of forked process
-int cpu_id;	// cpu id of child process
+int ncpu = 0;	// number of forked process
+int cpu_id = -1;	// cpu id of child process
+
+/* the cpu_id of owner process is equal to 0. */
+#define is_owner() (cpu_id==-1)
 
 struct log_info {
 	int c_pid;
@@ -119,29 +126,62 @@ struct log_info {
 	int prev_size;
 };
 
-struct log_info info[LKST_CPU_MAX];
+struct log_info info[LKST_CPU_MAX] = {
+	[0 ... LKST_CPU_MAX-1] = {
+		c_pid:		-1, 
+		buf_size:	0, 
+		limit_size:	0, 
+		total_size:	0, 
+		buf_id: 	{[0 ... INIT_BUF_NUM_MAX-1] = LKST_BUFFER_ID_VOID },
+		prev_id:	LKST_BUFFER_ID_VOID,
+		prev_size:	0}
+};
 
 lkst_maskset_id maskset_id_prev = LKST_MASKSET_ID_VOID;
 		// previous selected maskset id
-lkst_maskset_id maskset_id_daemon;
-		// daemon id of maskset 
+lkst_maskset_id maskset_id_daemon = LKST_MASKSET_ID_VOID;
+                // daemon id of maskset 
+
+void (*signal_reuse(int signum, void (*handler)(int)))(int);
+
+/* this is like BSD style signal(2) */
+void (*signal_reuse(int signum, void (*handler)(int)))(int) 
+{
+	struct sigaction sa, psa;
+
+	sa.sa_handler = handler;
+	if ( sigemptyset(&sa.sa_mask) < 0 ||
+	     sigaddset(&sa.sa_mask, signum) < 0 )
+		return SIG_ERR;
+	sa.sa_flags = 0;
+	if( sigaction(signum, &sa, &psa) < 0) 
+		return SIG_ERR;
+	else 
+		return psa.sa_handler;
+}
+
+void warnlog(const char *format, ...)
+{
+	va_list ap;
+	va_start(ap,format);
+	vsyslog(LOG_WARNING, format, ap);
+}
+void errlog(const char *format, ...)
+{
+	va_list ap;
+	va_start(ap,format);
+	vsyslog(LOG_ERR, format, ap);
+}
+
 
 int main(int argc, char** argv)
 {
-	int o_pid;	/* pid of the owner process */
-	int pid;	/* pid of the child process */
-  
-	if( (pid = fork()) < 0) {	/* ^Z */
-		printf("lkstlogd error: Unable to fork process\n");
-		exit(1);
-	} else if(pid > 0) {
-		exit(0);
-	}
-
 	parse_arg(argc, argv);
 
-	setsid();	/* become session leader */
-
+	if ( daemon(0,0) < 0) {
+		fprintf(stderr, "lkstlogd error: failed to daemonize\n");
+		return 1;
+	}
 	owner_daemon();
 
 	exit(0);
@@ -197,47 +237,57 @@ int owner_daemon()
 
 	FILE *pfd;
 
+	openlog("lkstlogd", LOG_PID, LOG_DAEMON);
+
+	/* check pid file */
+	if ( (pfd = fopen(pid_file, "r")) != NULL){
+		errlog( "%s exist, lkstlogd is already running.\n", pid_file);
+		exit(1);
+	}
+	
+	atexit(cleanup);
+	/* write pid file */
+	if ( (pfd = fopen(pid_file, "w")) == NULL){
+		errlog( "Can't open %s : %s\n", pid_file,strerror(errno));
+		pid_file[0]='\0';
+		exit(1);
+	}
+	if (!fprintf(pfd,"%d\n", getpid())) {
+		errlog( "Can't write %s : %s\n", pid_file,strerror(errno));
+		fclose(pfd);
+		exit(1);
+	}
+	fclose(pfd);
+	
+
+	devfd = open_dev();
 	/* Main loop */
 	while(1)
 	{
-		/* check pid file */
-		if ( (pfd = fopen(pid_file, "r")) != NULL){
-			fprintf(stderr, "lkstlogd error: lkstlogd is already started.\n");
-			exit(1);
-		}
-	
-		/* write pid file */
-		if ( (pfd = fopen(pid_file, "w")) == NULL){
-			fprintf(stderr, "lkstlogd error: Can't open %s.\n", pid_file);
-			exit(1);
-		}
-		if (!fprintf(pfd,"%d\n", getpid())) {
-			fprintf(stderr, "lkstlogd error: Can't write %s.\n", pid_file);
-			exit(1);
-		}
-		fclose(pfd);
-
-		devfd = open_dev();
 		/* set ncpu */
 		active_cpu(devfd, bufsize);
 		
-		init_buffer(devfd);
-
 		/* change maskset for daemon */
 		init_maskset(devfd);
-		close(devfd);
+
+		init_buffer(devfd);
 
 		/* set up sighandler */
-		if(signal(SIGHUP, sighup_handler) == SIG_ERR) exit(1);
-		if(signal(SIGINT, die_handler) == SIG_ERR) exit(1);
-		if(signal(SIGTERM, die_handler) == SIG_ERR) exit(1);
-		if(signal(SIGUSR1, par_snap_handler) == SIG_ERR) exit(1);
-		if(signal(SIGUSR2, par_switch_handler) == SIG_ERR) exit(1);
+		if(signal(SIGHUP, sighup_handler) == SIG_ERR ||
+		   signal(SIGINT, die_handler) == SIG_ERR    ||
+		   signal(SIGTERM, die_handler) == SIG_ERR   ||
+		   signal(SIGPIPE, die_handler) == SIG_ERR   ||
+		   signal_reuse(SIGUSR1, par_snap_handler) == SIG_ERR ||
+		   signal_reuse(SIGUSR2, par_switch_handler) == SIG_ERR) {
+			errlog( "Unable to fork process\n");
+			exit(1);
+		}
+		cpu_id = -1;
 
 		/* fork childlen */
 		for(i=0; i<ncpu; i++){
 			if( (info[i].c_pid = fork()) < 0) {
-				printf("lkstlogd error: Unable to fork process\n");
+				errlog( "Unable to fork process\n");
 				exit(1);
 			} else if(!info[i].c_pid) {
 				cpu_id = i;
@@ -250,10 +300,14 @@ int owner_daemon()
 		/* wait childlen */
 		for(i=0; i<ncpu; i++){
 			int status;
-			waitpid((pid_t)info[i].c_pid, &status, 0);
+			while( 0 > waitpid((pid_t)info[i].c_pid, &status, 0));
+			info[i].c_pid = -1;
 		}
 
-		cleanup();
+
+		restore_buffer(devfd);
+		restore_maskset(devfd);
+
 		if(restart == 0)
 			break;
 
@@ -261,36 +315,34 @@ int owner_daemon()
 		
 	}
 
+	close(devfd);
+	closelog();
+
 	return 0;
 }
 
 int child_daemon_main()
 {
-	struct sigaction sa;
 	char n[256];
 	FILE *tmpfp;
 
 	sprintf(n, "%s%02d.%01d", log_file, cpu_id, file_snum);
 	if ( (tmpfp = fopen(n, "w")) == NULL){
-		fprintf(stderr, "lkstlogd error: Can't open %s.\n", n);
+		errlog( "Can't open %s\n", n);
 		exit(1);
 	}
 	fclose(tmpfp);
 
 	/* set up sighandler */
-	sigemptyset(&sa.sa_mask);
-	sa.sa_flags = SA_NODEFER;
-	if(read_on == 0)
-		sa.sa_handler = child_snap_handler;
-	else
-		sa.sa_handler = child_end_snap_handler;
 
-	sigaction(SIGUSR1, &sa, NULL);
-
-	if(signal(SIGUSR2, child_switch_handler) == SIG_ERR) exit(1);
-	if(signal(SIGHUP, SIG_DFL) == SIG_ERR) exit(1);
-	if(signal(SIGINT, SIG_DFL) == SIG_ERR) exit(1);
-	if(signal(SIGTERM, child_die_handler) == SIG_ERR) exit(1);
+	if( signal_reuse(SIGUSR1, child_snap_flip_handler) == SIG_ERR ||
+	    signal_reuse(SIGUSR2, child_switch_handler) == SIG_ERR ||
+	    signal(SIGHUP, SIG_DFL) == SIG_ERR ||
+	    signal(SIGINT, SIG_DFL) == SIG_ERR ||
+	    signal(SIGTERM, child_die_handler) == SIG_ERR) {
+		errlog( "failed to register sighandlers\n");
+		exit(1);
+	}
 
 	term_flg = 0;
 
@@ -310,7 +362,7 @@ void chk_limit_and_seek(FILE *fd, int res_size){
 	if(info[cpu_id].total_size > info[cpu_id].limit_size) {
 		ret = fseek(fd, (off_t)LOG_HEADER_SIZE, SEEK_SET);
 		if(ret<0) {
-			fprintf(stderr, "lkstlogd error: Can't seek file pointer.\n");
+			errlog( "Can't seek file pointer.\n");
 			exit(1);
 		} else {
 			info[cpu_id].total_size = LOG_HEADER_SIZE + res_size;
@@ -335,13 +387,13 @@ void child_buf_read()
 
 	sprintf(n, "%s%02d.%01d", log_file, cpu_id, file_snum);
 	if ( (logfd = fopen(n, "w")) == NULL){
-		fprintf(stderr, "lkstlogd error: Can't open %s.\n", n);
+		errlog( "Can't open %s\n", n);
 		exit(1);
 	}
 
 	buffer = (struct lkst_log_record *) malloc(read_size);
 	if(!buffer){
-		fprintf(stderr, "lkstlogd error: Can't allocate buffer area.\n");
+		errlog( "Can't allocate buffer area\n");
 		exit(1);
 	}
 
@@ -475,7 +527,7 @@ int active_cpu(int devfd, int *bufsize)
 	retval = ioctl(devfd, LKST_IOC_BUFFER_LIST, &lp);
 	
 	if (retval) {
-		fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_BUFFER_LIST) error: %s\n", strerror(errno));
+		errlog( "ioctl(LKST_IOC_BUFFER_LIST) error: %s\n", strerror(errno));
 		exit(1);
 	}
 
@@ -500,6 +552,7 @@ void init_buffer(int devfd)
 	struct lkst_buffer_listparam lp;
 	struct lkst_buffer_listent listent[LKST_BUFFER_TBL_MAX];
 
+	if(devfd < 0)return;
 	for (i=0; i<ncpu; i++) {
 		for (j=0; j<init_buf_num; j++){
 			param.id = LKST_BUFFER_ID_VOID;
@@ -508,7 +561,7 @@ void init_buffer(int devfd)
 
 			ret = ioctl(devfd, LKST_IOC_BUFFER_CREATE, &param);
 			if (ret) {
-				fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_BUFFER_CREATE) error: %s\n", strerror(errno));
+				errlog( "ioctl(LKST_IOC_BUFFER_CREATE) error: %s\n", strerror(errno));
 				exit(1);
 			}
 
@@ -529,18 +582,16 @@ void init_buffer(int devfd)
 			}
 		}
 	}
-
 }
 
-void restore_buffer(void)
+void restore_buffer(int devfd)
 {
-	int devfd;
 	int ret, i, j;
  	struct lkst_buffer_param param;
 
-	devfd = open_dev();
-	
-	for (i=0; i<ncpu; i++) {
+	if(devfd < 0)return;
+	for (i=0; i<LKST_CPU_MAX ; i++) {
+		if ( info[i].prev_id == LKST_BUFFER_ID_VOID ) continue;
 		do {
 			ioctl(devfd, LKST_IOC_BUFFER_SHIFT, i);
 			ioctl(devfd, LKST_IOC_BUFFER_DELETE, info[i].prev_id);
@@ -548,19 +599,23 @@ void restore_buffer(void)
 			param.size = info[i].prev_size;
 			param.cpu = i;
 			ret = ioctl(devfd, LKST_IOC_BUFFER_CREATE, &param);
-		} while(ret);
+		} while(ret < 0);
+		info[i].prev_id = LKST_BUFFER_ID_VOID;
 
-		for (j=0; j<init_buf_num; j++){
+		for (j=0; j<INIT_BUF_NUM_MAX; j++){
+			if (info[i].buf_id[j] == LKST_BUFFER_ID_VOID ) continue;
 		retry:
 			ret = ioctl(devfd, LKST_IOC_BUFFER_DELETE, info[i].buf_id[j]);
-			if ( ret == 0 ) continue;
-			if (errno == EBUSY) {
-				ioctl(devfd, LKST_IOC_BUFFER_SHIFT, i);
-				goto retry;
-			} else {
-				fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_BUFFER_DELETE) error: %s\n", strerror(errno));
-				exit(1);
+			if ( ret != 0 ) {
+				if (errno == EBUSY) {
+					ioctl(devfd, LKST_IOC_BUFFER_SHIFT, i);
+					goto retry;
+				} else {
+					errlog("Cannot delete buffer id=%d : %s\n", 
+						info[i].buf_id[j], strerror(errno));
+				}
 			}
+			info[i].buf_id[j] = LKST_BUFFER_ID_VOID;
 		}
 	}
 }
@@ -572,6 +627,7 @@ void init_maskset(int devfd)
 	struct lkst_maskset_body maskset;
 	struct lkst_maskset_entry *entry;
 
+	if(devfd < 0)return;
 	/* read current maskset */
 	param.id = LKST_MASKSET_ID_VOID;
 	param.maskset_size = LKST_MASKSET_SIZE(LKST_MASKSET_TABLE_LEN_MAX);
@@ -579,33 +635,41 @@ void init_maskset(int devfd)
 
 	retval = ioctl(devfd, LKST_IOC_MASKSET_READ, &param);
 	if (retval) {
-		fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_MASKSET_READ) error: %s\n", strerror(errno));
+		errlog( "ioctl(LKST_IOC_MASKSET_READ) error: %s\n", strerror(errno));
 		exit(1);
 	}
 	maskset_id_prev = param.id;
 
+	retval = -1;
 	/* change buffer overflow event handler */
 	for (i = 0; i < maskset.len; i++) {
 		entry = &maskset.entry[i];
 		if (entry->event_type == LKST_ETYPE_LKST_BUFF_OVFLOW) {
 			entry->id = LKST_EVHANDLER_ID_BUFFER_SHIFT_DW;
-			break;
+			retval = i;
+		}
+		/* Currently, daemon can not log the lock events. */
+		if ( (entry->event_type>>4) == (LKST_ETYPE_LK_SPINLOCK>>4)){
+			entry->id = LKST_EVHANDLER_ID_VOID;
 		}
 	}
-	if (i == maskset.len) {
-		fprintf(stderr, "lkstlogd error: Cannot initialize maskset for daemon!\n");
+	if (retval == -1) {
+		errlog( "Cannot initialize maskset for daemon!\n");
 		exit(1);
 	}
 
 	/* write new maskset */
 	param.id = LKST_MASKSET_ID_VOID;
-	if (strlen(maskset.name) < (LKST_MASKSET_NAME_LEN - 8))
-		strncat(maskset.name, "_DAEMON", 7);
+	if (strlen(maskset.name) >= (LKST_MASKSET_NAME_LEN - 8)){
+		maskset.name[LKST_MASKSET_NAME_LEN - 8] = '\0';
+	}
+	strncat(maskset.name, "_DAEMON", 7);
+		
 	retval = ioctl(devfd, LKST_IOC_MASKSET_WRITE, &param);
 	if (retval) {
-		fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_MASKSET_WRITE) error: %s\n", strerror(errno));
-		fprintf(stderr, "lkstlogd error: Maybe, lock events are recording, or maskset id is full.\n"); 
-		del_pid_file();
+		errlog( "ioctl(LKST_IOC_MASKSET_WRITE) error: %s\n", strerror(errno));
+		errlog( "Maybe, lock events are recording, or maskset id is full.\n"); 
+		exit(1);
 	}
 
 	maskset_id_daemon = param.id;
@@ -613,33 +677,33 @@ void init_maskset(int devfd)
 	/* change maskset */
 	retval = ioctl(devfd, LKST_IOC_MASKSET_SET, maskset_id_daemon);
 	if (retval) {
-		fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_MASKSET_SET) error: %s\n", strerror(errno));
+		errlog( "ioctl(LKST_IOC_MASKSET_SET) error: %s\n", strerror(errno));
 		exit(1);
 	}
 }
 
-void restore_maskset(void)
+void restore_maskset(int devfd)
 {
-	int devfd;
 	int retval;
 
+	if(devfd < 0 )return;
 	if (maskset_id_prev != LKST_MASKSET_ID_VOID) {
-		devfd = open_dev();
 		/* restore maskset */
 		retval = ioctl(devfd, LKST_IOC_MASKSET_SET, maskset_id_prev);
 		if (retval) {
-			fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_MASKSET_SET) error: %s\n", strerror(errno));
-			fprintf(stderr, "lkstlogd error: Cannot restore maskset!\n");
-			exit(1);
+			errlog( "ioctl(LKST_IOC_MASKSET_SET) error: %s\n", strerror(errno));
+			errlog( "Cannot restore maskset!\n");
 		}
+		maskset_id_prev = LKST_MASKSET_ID_VOID;
+	}
+	if (maskset_id_daemon != LKST_MASKSET_ID_VOID) {
 		/* delete daemon maskset */
 		retval = ioctl(devfd, LKST_IOC_MASKSET_DELETE, maskset_id_daemon);
 		if (retval) {
-			fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_MASKSET_DELETE) error: %s\n", strerror(errno));
-			fprintf(stderr, "lkstlogd error: Cannot delete maskset for daemon!\n");
-			exit(1);
+			errlog( "ioctl(LKST_IOC_MASKSET_DELETE) error: %s\n", strerror(errno));
+			errlog( "Cannot delete maskset for daemon!\n");
 		}
-		close(devfd);
+		maskset_id_daemon = LKST_MASKSET_ID_VOID;
 	}
 }
 
@@ -648,9 +712,11 @@ int buf_read(int devfd, struct lkst_log_record *buffer, int read_size)
 	int result;
 
 	result = read(devfd, buffer, read_size);
-	if(result < 0){
-		fprintf(stderr, "lkstlogd error: lkst device read() error: %s\n", strerror(errno));
-		perror("read");
+	if(result < 0 ){
+		if(errno == EAGAIN )
+			warnlog( "warning : skip one (or more) buffer(s)\n");
+		else		
+			errlog( "lkst device read() error: %s\n", strerror(errno));
 	}
   
 	return result;
@@ -661,16 +727,19 @@ int buffer_read_size(int buf_size)
 	return buf_size * sizeof(struct lkst_log_record) / LKST_SIZEOF_LKST_EVENT_RECORD;
 }
 
-void cleanup()
+void cleanup(void)
 {
-
-	restore_maskset();
-	restore_buffer();
-
+	int devfd;
+	if (! is_owner()) return;
+	kill_children(SIGTERM);
 	del_pid_file();
+	devfd = open_dev();
+	restore_buffer(devfd);
+	restore_maskset(devfd);
+	close(devfd);
 }
 
-void sighup_handler()
+void sighup_handler(int s)
 {
 	kill_children(SIGTERM);
 	restart = 1;
@@ -688,23 +757,19 @@ void child_die_handler(int sigid)
 	term_flg = 1;
 }
 
-void par_snap_handler()
+void par_snap_handler(int s)
 {
 	kill_children(SIGUSR1);
-	signal(SIGUSR1, par_snap_handler);
-
 	return;
 }
 
-void par_switch_handler()
+void par_switch_handler(int s)
 {
 	kill_children(SIGUSR2);
-	signal(SIGUSR2, par_switch_handler);
-
 	return;
 }
 
-void child_switch_handler()
+void child_switch_handler(int s)
 {
 	FILE *fd;
 	char n[256];
@@ -712,7 +777,7 @@ void child_switch_handler()
 	file_snum++;
 	sprintf(n, "%s%02d.%01d", log_file, cpu_id, file_snum);
 	if ( (fd = fopen(n, "w")) == NULL){
-		fprintf(stderr, "lkstlogd error: Can't open %s.\n", n);
+		errlog( "Can't open %s\n", n);
 		exit(1);
 	}
 
@@ -728,38 +793,14 @@ void child_switch_handler()
 		change_fd = 1;
 		logfd_c=fd;
 	}
-
-	signal(SIGUSR2, child_switch_handler);
 }
 
-void child_end_snap_handler()
+void child_snap_flip_handler(int s)
 {
-	struct sigaction sa;
-
-	/* set up sighandler */
-	sigemptyset(&sa.sa_mask);
-	sa.sa_flags = SA_NODEFER;
-	sa.sa_handler = child_snap_handler;
-	sigaction(SIGUSR1, &sa, NULL);
-
-	read_on = 0;
-
-	return;
-}
-
-void child_snap_handler()
-{
-	struct sigaction sa;
-
-	/* set up sighandler */
-	sigemptyset(&sa.sa_mask);
-	sa.sa_flags = 0;
-	sa.sa_handler = child_end_snap_handler;
-	sigaction(SIGUSR1, &sa, NULL);
-
-	read_on = 1;
-
-	return;
+	if(read_on)
+		read_on = 0;
+	else
+		read_on = 1;
 }
 
 void buffer_setrmod(int devfd)
@@ -772,7 +813,7 @@ void buffer_setrmod(int devfd)
 
 	ret = ioctl(devfd, LKST_IOC_BUFFER_SETRMOD, &sp);
 	if( ret ) {
-		fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_BUFFER_SETRMOD) error: %s\n", strerror(errno));
+		errlog( "ioctl(LKST_IOC_BUFFER_SETRMOD) error: %s\n", strerror(errno));
 		exit(1);
 	}
 
@@ -795,7 +836,7 @@ void buffer_ring(int devfd, int ring)
         retval = ioctl(devfd, LKST_IOC_BUFFER_RING, &rp); /* buffer_ring */
 
         if (retval) {
-                fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_BUFFER_RING) error: %s\n", strerror(errno));
+                errlog( "ioctl(LKST_IOC_BUFFER_RING) error: %s\n", strerror(errno));
                 exit(1);
         }
 }
@@ -805,16 +846,18 @@ int kill_children(int sigid)
 	int i;
 
 	for(i=0; i<ncpu; i++)
-		kill(info[i].c_pid, sigid);
+		if(info[i].c_pid > 0) kill(info[i].c_pid, sigid);
 
 	return 0;
 }
 
 int del_pid_file(){
 	/* pid_file remove */
+	if(pid_file[0]=='\0') return 0;
 	if ( unlink((char *)pid_file) != 0){
-		fprintf(stderr, "lkstlogd error: Can't remove %s.\n", pid_file);
+		errlog( "Can't remove %s.\n", pid_file);
 	}
+	pid_file[0]='\0';
 	return 0;
 }
 
