@@ -74,7 +74,7 @@ void usage(void)
 	printf("Usage:\n\t%s command [option(s)]\n\n", NAME);
 	printf("<COMMAND>\n");
 	printf("all\t\tOutput a list of masksets and display content of all masksets.\n");
-	printf("exchange/xchg\tChange a currently selected maskset and display id of old one.\n");
+	/*printf("exchange/xchg\tChange a currently selected maskset and display id of old one.\n");*/
 	printf("\t\t<OPTION>\n");
 	printf("\t\t-m maskset_id\n");
 	printf("delete/del\tDelete a maskset.\n");
@@ -82,7 +82,7 @@ void usage(void)
 	printf("read\t\tOutput a content of maskset(s).\n");
 	printf("\t\t<OPTION>\n");
 	printf("\t\t[-m maskset_id] [-A] [-a] [-d]\n");
-	printf("set\t\tSame as 'exchange' except for not displaying id of old maskset.\n");
+	printf("set\t\tChange to sepecified maskset.\n");
 	printf("\t\t<OPTION>\n");
 	printf("\t\t-m maskset_id\n");
 	printf("write\t\tWrite a new maskset.\n");
